"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compileToJs = exports.compileToRegexp = void 0;
const formula_1 = require("@datasco/formula");
/**
 * Compile formula to RegExp text
 * @param formula formula code
 * @returns result of compilation
 */
const compileToRegexp = (formula) => {
    // 1. Lexing
    const lexed = new formula_1.Lexer().lex(formula);
    // 2. Parsing
    const parseResult = new formula_1.Parser().parse(lexed);
    // 3. Transpilating
    return {
        type: "js",
        result: parseResult.program.serialize(parseResult.context),
        context: parseResult.context,
    };
};
exports.compileToRegexp = compileToRegexp;
/**
 * Compile formula to JS code
 * @param formula formula code
 * @returns result of compilation
 */
const compileToJs = (formula, exportAs = "regexp") => {
    const { result, context } = (0, exports.compileToRegexp)(formula);
    return {
        type: "regexp",
        result: /*js*/ `
            export const ${exportAs} = new RegExp(/${result}/g);
        `,
        context,
    };
};
exports.compileToJs = compileToJs;
