"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Lexer_1 = __importDefault(require("./parser/Lexer"));
const Parser_1 = __importDefault(require("./parser/Parser"));
const readFile_1 = require("./helpers/readFile");
const writeFile_1 = require("./helpers/writeFile");
const chalk_1 = __importDefault(require("chalk"));
(() => __awaiter(void 0, void 0, void 0, function* () {
    let formulaContent = yield (0, readFile_1.readFile)('src/test/in.formula');
    // Lex
    let lexed = new Lexer_1.default(true).lex(formulaContent);
    console.log('--------------');
    console.log(chalk_1.default.yellow('LEXED'), lexed);
    console.log('--------------');
    // Parse
    let parseResult = new Parser_1.default(true).parse(lexed);
    // Serialize
    let result = parseResult.program.serialize(parseResult.context);
    console.log('--------------');
    console.log(chalk_1.default.blue('RESULT'), result);
    console.log(chalk_1.default.blue('CONTEXT'), parseResult.context);
    console.log('--------------');
    // Test
    console.log(chalk_1.default.blue('TESTING'));
    const textSuccess = chalk_1.default.green;
    const textError = chalk_1.default.red;
    let testNumber = 1;
    for (const test of parseResult.context.test) {
        const success = new RegExp(result).test(test.value);
        const message = `${success ? '✓' : '✖'} Test ${testNumber}: ${test.value}`;
        console.log(' '.repeat(4) +
            (success
                ? textSuccess(message)
                : textError(message)));
        testNumber++;
    }
    console.log('--------------');
    // Save
    yield (0, writeFile_1.writeFile)('src/test/out.txt', result);
}))();
