"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const TokenType_1 = __importDefault(require("./type/TokenType"));
const StatementContext_1 = __importDefault(require("./statement/StatementContext"));
const ProgramStatement_1 = __importDefault(require("./statement/statements/ProgramStatement"));
const MatchStatement_1 = __importDefault(require("./statement/statements/MatchStatement"));
const GroupStatement_1 = __importDefault(require("./statement/statements/GroupStatement"));
const IfElseStatement_1 = __importDefault(require("./statement/statements/IfElseStatement"));
const DefineStatement_1 = __importDefault(require("./statement/statements/DefineStatement"));
const TestStatement_1 = __importDefault(require("./statement/statements/TestStatement"));
class Parser {
    constructor(debug = false) {
        this._debug = debug;
    }
    log(...messages) {
        if (!this._debug)
            return;
        console.log(...messages);
    }
    parse(tokens) {
        const self = this;
        const out = [];
        const context = new StatementContext_1.default();
        let currPos = 0;
        function next() {
            currPos++;
        }
        function prev() {
            currPos--;
        }
        function expect(type) {
            var _a;
            let tok = tokens[currPos];
            if ((tok === null || tok === void 0 ? void 0 : tok.type) === type) {
                return true;
            }
            else {
                throw new Error(`Expected "${type}" token, but got: "${tok === null || tok === void 0 ? void 0 : tok.type}" (At line ${(_a = tok === null || tok === void 0 ? void 0 : tok.pos) === null || _a === void 0 ? void 0 : _a.start.line})`);
            }
        }
        function is(type) {
            var _a;
            return ((_a = tokens[currPos]) === null || _a === void 0 ? void 0 : _a.type) === type;
        }
        function expectNext(type) {
            next();
            return expect(type);
        }
        function isNext(type) {
            var _a;
            return ((_a = tokens[currPos + 1]) === null || _a === void 0 ? void 0 : _a.type) === type;
        }
        // Statements
        function parse() {
            var _a;
            self.log('IN PARSE', currPos, tokens[currPos]);
            let currToken = tokens[currPos];
            // Match
            if ((currToken === null || currToken === void 0 ? void 0 : currToken.type) === TokenType_1.default.Match) {
                self.log('IN PARSE PARSING MATCH', currPos);
                let res = parseMatch();
                self.log('IN PARSE PARSED MATCH', res, currPos, tokens[currPos]);
                return res;
            }
            // Group
            else if ((currToken === null || currToken === void 0 ? void 0 : currToken.type) === TokenType_1.default.Group) {
                self.log('IN PARSE PARSING GROUP');
                let res = parseGroup();
                self.log('IN PARSE PARSED GROUP', res, currPos, tokens[currPos]);
                return res;
            }
            // If-else
            else if ((currToken === null || currToken === void 0 ? void 0 : currToken.type) === TokenType_1.default.If) {
                self.log('IN PARSE PARSING IF-ELSE');
                let res = parseIfElse();
                self.log('IN PARSE PARSED IF-ELSE', res, currPos, tokens[currPos]);
                return res;
            }
            // Define
            else if ((currToken === null || currToken === void 0 ? void 0 : currToken.type) === TokenType_1.default.Define) {
                self.log('IN PARSE PARSING DEFINE');
                let res = parseDefine();
                self.log('IN PARSE PARSED DEFINE', res, currPos, tokens[currPos]);
                return res;
            }
            // Test
            else if ((currToken === null || currToken === void 0 ? void 0 : currToken.type) === TokenType_1.default.Test) {
                self.log('IN PARSE PARSING TEST');
                let res = parseTest();
                self.log('IN PARSE PARSED TEST', res, currPos, tokens[currPos]);
                return res;
            }
            else if (currToken) {
                throw new Error(`Unknown statement. Expected "match" or "group" or "if", but got: "${currToken === null || currToken === void 0 ? void 0 : currToken.type}" (At line ${(_a = currToken === null || currToken === void 0 ? void 0 : currToken.pos) === null || _a === void 0 ? void 0 : _a.start.line})`);
            }
            return null;
        }
        function parseBody(until) {
            let result = [];
            let currToken = tokens[currPos];
            self.log('PREV LOOP', currToken);
            do {
                currToken = tokens[currPos];
                if ((currToken === null || currToken === void 0 ? void 0 : currToken.type) === until || !currToken) {
                    break;
                }
                self.log('BEFORE PARSE', currToken, currPos);
                // Parse statement
                let stmt = parse();
                // Add
                if (stmt) {
                    result.push(stmt);
                }
                self.log('AFTER PARSE', currToken, currPos);
            } while ((currToken === null || currToken === void 0 ? void 0 : currToken.type) !== until || !currToken);
            if ((currToken === null || currToken === void 0 ? void 0 : currToken.type) !== until) {
                prev();
            }
            self.log('AFTER LOOP', tokens[currPos]);
            return result;
        }
        // (min,max)/+ and ?
        function parseMatchParams() {
            let token = tokens[currPos];
            let params = {
                optional: false
            };
            // Multiple?
            if ((token === null || token === void 0 ? void 0 : token.type) === TokenType_1.default.Mutiple) {
                // From 1 to unlimited
                params.amount = {};
                // Next
                next();
                token = tokens[currPos];
            }
            else if ((token === null || token === void 0 ? void 0 : token.type) === TokenType_1.default.ArgStart) {
                let min = undefined;
                let max = undefined;
                // Next
                next();
                token = tokens[currPos];
                // Minimal
                if ((token === null || token === void 0 ? void 0 : token.type) === TokenType_1.default.LiteralNumber) {
                    min = parseInt(token.value);
                    next();
                }
                else {
                    throw new Error('Expected number (amount of matches), but got: ' + token);
                }
                token = tokens[currPos];
                // Maximal
                if (min !== undefined && (token === null || token === void 0 ? void 0 : token.type) === TokenType_1.default.Comma) {
                    next();
                    token = tokens[currPos];
                    if ((token === null || token === void 0 ? void 0 : token.type) === TokenType_1.default.LiteralNumber) {
                        max = parseInt(token.value);
                        next();
                    }
                    else {
                        throw new Error('Expected number (amount of max matches), but got: ' + token);
                    }
                }
                // )
                if (expect(TokenType_1.default.ArgEnd)) {
                    next();
                    token = tokens[currPos];
                    params.amount = { min, max };
                }
            }
            // Optional?
            if ((token === null || token === void 0 ? void 0 : token.type) === TokenType_1.default.Optional) {
                params.optional = true;
                // Next
                next();
                token = tokens[currPos];
            }
            return params;
        }
        // Match
        function parseMatchPrimitive() {
            var _a, _b;
            let token = tokens[currPos];
            let primitive;
            // Lookbehind
            let isLookAhead = false;
            if ((token === null || token === void 0 ? void 0 : token.type) === TokenType_1.default.Pseudo) {
                isLookAhead = true;
                // Skip this token
                next();
                token = tokens[currPos];
            }
            // Variable
            if ((token === null || token === void 0 ? void 0 : token.type) === TokenType_1.default.Name) {
                primitive = {
                    type: 'define',
                    value: token.value
                };
                next();
                token = tokens[currPos];
            }
            // Literal
            else if ((token === null || token === void 0 ? void 0 : token.type) === TokenType_1.default.LiteralString) {
                primitive = {
                    type: 'literal',
                    value: token.value
                };
                next();
                token = tokens[currPos];
            }
            else {
                throw new Error(`Expected match primitive (variable name or value), but got: "${token === null || token === void 0 ? void 0 : token.type}" (At line ${(_a = token === null || token === void 0 ? void 0 : token.pos) === null || _a === void 0 ? void 0 : _a.start.line})`);
            }
            // Lookbehind
            if (isLookAhead) {
                primitive.pseudo = 'ahead';
            }
            // Lookahead
            if ((token === null || token === void 0 ? void 0 : token.type) === TokenType_1.default.Pseudo) {
                next();
                // Without lookbehind
                if (!isLookAhead) {
                    primitive.pseudo = 'behind';
                }
                else {
                    throw new Error(`Unexpected "..." lookbehind, you cannot have lookahead and lookbehind at the same time (At line ${(_b = token === null || token === void 0 ? void 0 : token.pos) === null || _b === void 0 ? void 0 : _b.start.line})`);
                }
            }
            return primitive;
        }
        function parseMatch() {
            var _a;
            next();
            let params = parseMatchParams();
            let primitives = [parseMatchPrimitive()];
            // While we got or delimiter
            while (((_a = tokens[currPos]) === null || _a === void 0 ? void 0 : _a.type) === TokenType_1.default.Or) {
                next();
                primitives.push(parseMatchPrimitive());
            }
            return new MatchStatement_1.default("or", primitives, params);
        }
        // Group
        function parseGroup() {
            next();
            let params = parseMatchParams();
            // Name
            let name = undefined;
            if (is(TokenType_1.default.Name)) {
                name = tokens[currPos].value;
                next();
            }
            // Anonymous name
            else if (is(TokenType_1.default.Anonymous)) {
                next();
                name = null;
            }
            // {
            expect(TokenType_1.default.GroupStart);
            next();
            // Body
            let body = parseBody(TokenType_1.default.GroupEnd);
            let group = new GroupStatement_1.default(body, params, name);
            // }
            expect(TokenType_1.default.GroupEnd);
            next();
            self.log('PARSED GROUP');
            return group;
        }
        // If-else
        function parseIfElse() {
            // (
            expectNext(TokenType_1.default.ArgStart);
            next();
            // Condition
            let condition = parseBody(TokenType_1.default.ArgEnd);
            // )
            expect(TokenType_1.default.ArgEnd);
            // {
            expectNext(TokenType_1.default.GroupStart);
            next();
            // Then
            let thenBranch = parseBody(TokenType_1.default.GroupEnd);
            // }
            expect(TokenType_1.default.GroupEnd);
            next();
            let elseBranch = [];
            if (is(TokenType_1.default.Else)) {
                // {
                expectNext(TokenType_1.default.GroupStart);
                next();
                // Else
                elseBranch = parseBody(TokenType_1.default.GroupEnd);
                // }
                expect(TokenType_1.default.GroupEnd);
                next();
            }
            self.log('PARSED IF-ELSE');
            return new IfElseStatement_1.default(condition, thenBranch, elseBranch);
        }
        // Define
        function parseDefine() {
            var _a, _b, _c, _d, _e, _f;
            next();
            // Name
            let name = undefined;
            if (is(TokenType_1.default.Name)) {
                name = tokens[currPos].value;
                next();
            }
            else {
                throw new Error(`Unknown statement. Expected name, but got: "${(_a = tokens[currPos]) === null || _a === void 0 ? void 0 : _a.type}" (At line ${(_c = (_b = tokens[currPos]) === null || _b === void 0 ? void 0 : _b.pos) === null || _c === void 0 ? void 0 : _c.start.line})`);
            }
            // Value
            let value = parse();
            if (value) {
                let stmt = new DefineStatement_1.default(value);
                // Add to context
                context.define[name] = stmt;
                return stmt;
            }
            throw new Error(`Expected value, but got: "${(_d = tokens[currPos]) === null || _d === void 0 ? void 0 : _d.type}" (At line ${(_f = (_e = tokens[currPos]) === null || _e === void 0 ? void 0 : _e.pos) === null || _f === void 0 ? void 0 : _f.start.line})`);
        }
        // Test
        function parseTest() {
            var _a, _b, _c;
            next();
            // Value
            let value = undefined;
            if (is(TokenType_1.default.LiteralString)) {
                value = tokens[currPos].value;
                next();
            }
            else {
                throw new Error(`Expected value, but got: "${(_a = tokens[currPos]) === null || _a === void 0 ? void 0 : _a.type}" (At line ${(_c = (_b = tokens[currPos]) === null || _b === void 0 ? void 0 : _b.pos) === null || _c === void 0 ? void 0 : _c.start.line})`);
            }
            let stmt = new TestStatement_1.default(value);
            // Add to context
            context.test.push(stmt);
            return stmt;
        }
        while (currPos < tokens.length) {
            if (tokens[currPos + 1]) {
                let stmt = parse();
                if (stmt)
                    out.push(stmt);
            }
            else {
                break;
            }
        }
        return {
            program: new ProgramStatement_1.default(out),
            context
        };
    }
}
exports.default = Parser;
