import IStatement from "./IStatement";
import StatementContext from "./StatementContext";
import StatementType from "./StatementType";
declare abstract class IParentStatement implements IStatement {
    type: StatementType;
    children: IStatement[];
    constructor(type: StatementType, children: IStatement[]);
    abstract serialize(ctx: StatementContext): string;
}
declare class ProgramStatement extends IParentStatement {
    constructor(children: IStatement[]);
    serialize(ctx: StatementContext): string;
}
declare type MatcherAmount = {
    min: number;
    max?: number;
};
declare type MatcherParams = {
    optional: boolean;
    amount?: MatcherAmount;
};
declare class GroupStatement extends IParentStatement {
    named?: string | null;
    params: MatcherParams;
    constructor(children: IStatement[], params: MatcherParams, named?: string | null);
    serialize(ctx: StatementContext): string;
}
declare class IfElseStatement implements IStatement {
    type: StatementType;
    condition: IStatement[];
    branchIf: IStatement[];
    branchElse: IStatement[];
    constructor(condition: IStatement[], branchIf: IStatement[], branchElse: IStatement[]);
    serialize(ctx: StatementContext): string;
}
declare class DefineStatement implements IStatement {
    type: StatementType;
    value: IStatement;
    constructor(value: IStatement);
    serialize(ctx: StatementContext): string;
}
declare type SpecialMatcher = {
    of: string;
};
declare type LiteralMatcher = string;
declare type Matcher = SpecialMatcher | LiteralMatcher;
declare class MatchStatement implements IStatement {
    type: StatementType;
    values: Matcher[];
    params: MatcherParams;
    constructor(values: Matcher[], params?: MatcherParams);
    private serializeOne;
    serialize(ctx: StatementContext): string;
}
export { ProgramStatement, GroupStatement, MatchStatement, IfElseStatement, DefineStatement };
