"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const compileParams_1 = __importDefault(require("../../helpers/compileParams"));
const StatementType_1 = __importDefault(require("../StatementType"));
const Operators = {
    "or": "|"
};
class MatchStatement {
    constructor(operator, values, params = { optional: false }) {
        this.type = StatementType_1.default.Match;
        this.operator = operator;
        this.values = values;
        this.params = params;
    }
    serializeOne(ctx, primitive) {
        let regexResult = '';
        // Special
        if (primitive.type === 'define') {
            // Constants
            switch (primitive.value) {
                case 'START': {
                    regexResult = '^';
                    break;
                }
                case 'END': {
                    regexResult = '$';
                    break;
                }
                case 'DIGIT': {
                    regexResult = '\\d';
                    break;
                }
                case 'NONDIGIT': {
                    regexResult = '\\D';
                    break;
                }
                case 'LETTER': {
                    regexResult = '[a-zA-Z]';
                    break;
                }
                case 'WORD': {
                    regexResult = '\\w';
                    break;
                }
                case 'NONWORD': {
                    regexResult = '\\W';
                    break;
                }
                case 'ANY': {
                    regexResult = '.';
                    break;
                }
                case 'SPACE': {
                    regexResult = '\\s';
                    break;
                }
            }
            // Definitions
            if (ctx.define[primitive.value]) {
                regexResult = ctx.define[primitive.value].value.serialize(ctx);
            }
            else {
                throw new Error(`Unexpected variable "${primitive.value}", there's no definition for that variable.`);
            }
        }
        // Range of characters
        else if (primitive.value.startsWith('[') && primitive.value.endsWith(']')) {
            regexResult = primitive.value;
        }
        // Literal
        else {
            regexResult = primitive.value.replace(/\-/g, '\\-').replace(/\./g, '\\.');
        }
        // Lookahead
        if (primitive.pseudo === 'ahead') {
            regexResult = `(?=${regexResult})`;
        }
        // Lookbehind
        else if (primitive.pseudo === 'behind') {
            regexResult = `(?<=${regexResult})`;
        }
        return regexResult;
    }
    serialize(ctx) {
        let result = null;
        // Should values be packed into anonymous group? (For splitting with operator)
        let doPacking = this.values.length > 1 && this.values.every(primitive => primitive.type === 'literal' ? primitive.value.length > 1 : true);
        // Concat all values and split them with operator
        result = this.values.map(value => doPacking ? '(?:' + this.serializeOne(ctx, value) + ')'
            : this.serializeOne(ctx, value)).join(Operators[this.operator]);
        return result !== null
            ? (0, compileParams_1.default)(result, this.params)
            : '';
    }
}
exports.default = MatchStatement;
